/* 
 * PPRacer 
 * Copyright (C) 2004-2005 Volker Stroebel <volker@planetpenguin.de>
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "splash_screen.h"

#include "gl_util.h"
#include "fps.h"
#include "loop.h"
#include "render_util.h"
#include "textures.h"
#include "game_config.h"

#include "ppgltk/ui_mgr.h"
#include "ppgltk/ui_snow.h"

#include "ppgltk/audio/audio.h"

#define COORD_OFFSET_AMT -0.5
static char *logo_bindings[4] = { "splash_screen_tl",
				  "splash_screen_bl",
				  "splash_screen_tr",
				  "splash_screen_br" };

				  
SplashScreen::SplashScreen()
{
	init_ui_snow();
    play_music( "splash_screen" );
    reshape( getparam_x_resolution(), getparam_y_resolution() );
}

SplashScreen::~SplashScreen()
{
    //Free textures
    for (unsigned int i=0; i<sizeof(logo_bindings)/sizeof(logo_bindings[0]); i++) {
			unbind_texture( logo_bindings[i] );
    }
	flush_textures();
}

void
SplashScreen::loop(float timeStep)
{
	update_audio();

    clear_rendering_context();

    set_gl_options( GUI );

    UIMgr.setupDisplay();

	drawSnow(timeStep);

    drawLogo();

    UIMgr.draw();

    reshape( getparam_x_resolution(), getparam_y_resolution() );

    winsys_swap_buffers();	
}


bool
SplashScreen::mouseButtonReleaseEvent(int button, int x, int y)
{
	set_game_mode( GAME_TYPE_SELECT );
	winsys_post_redisplay();
	return true;
}

bool
SplashScreen::keyReleaseEvent(SDLKey key)
{
	set_game_mode( GAME_TYPE_SELECT );
	winsys_post_redisplay();
	return true;
}


void
SplashScreen::drawLogo()
{
    GLuint texid[4];
    int xoffsets[4] = { -1, -1, 0, 0 };
    int yoffsets[4] = { 0, -1, 0, -1 };
    pp::Vec2d ll, ur;
    GLint w, h;
    int i;

    glEnable( GL_TEXTURE_2D );

    for (i=0; i<4; i++) {
	if ( ! get_texture_binding( logo_bindings[i], &texid[i] ) ) {
	    return;
	}
    }

    glColor4f( 1.0, 1.0, 1.0, 1.0 );

    for (i=0; i<4; i++) {
	glBindTexture( GL_TEXTURE_2D, texid[i] );

	glGetTexLevelParameteriv( GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &w );
	glGetTexLevelParameteriv( GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &h );

	ll.x = getparam_x_resolution()/2 + xoffsets[i]*w;
	ll.y = getparam_y_resolution()/2 + yoffsets[i]*h;
	ur.x = ll.x + w;
	ur.y = ll.y + h;


	glBegin( GL_QUADS );
	{
	    glTexCoord2f( 0.0, 0.0 );
	    glVertex2f( ll.x, ll.y );
	    glTexCoord2f( 1.0, 0.0 );
	    glVertex2f( ur.x, ll.y );
	    glTexCoord2f( 1.0, 1.0 );
	    glVertex2f( ur.x, ur.y );
	    glTexCoord2f( 0.0, 1.0 );
	    glVertex2f( ll.x, ur.y );
	}
	glEnd();
    }

}
